package de.fitko.fitconnect.dependency;

import com.google.inject.AbstractModule;
import com.google.inject.Provides;
import com.google.inject.Singleton;
import com.google.inject.name.Named;
import com.google.inject.name.Names;
import de.fitko.fitconnect.api.services.Sender;
import de.fitko.fitconnect.api.services.Subscriber;
import de.fitko.fitconnect.api.services.auth.OAuthService;
import de.fitko.fitconnect.api.services.crypto.CryptoService;
import de.fitko.fitconnect.api.services.metadata.MetadataService;
import de.fitko.fitconnect.api.services.validation.CertificateValidator;
import de.fitko.fitconnect.api.services.validation.MetadataValidator;
import de.fitko.fitconnect.impl.SubmissionSender;
import de.fitko.fitconnect.impl.SubmissionSubscriber;
import de.fitko.fitconnect.impl.auth.DefaultOAuthService;
import de.fitko.fitconnect.impl.crypto.JWECryptoService;
import de.fitko.fitconnect.impl.http.ProxyConfig;
import de.fitko.fitconnect.impl.metadata.MetadataUploadService;
import de.fitko.fitconnect.impl.validation.KeyValidator;
import de.fitko.fitconnect.impl.validation.MetadataSubmissionValidator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.client.RestTemplate;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

public class SdkModule extends AbstractModule {

    private static final Logger logger = LoggerFactory.getLogger(SdkModule.class);
    public static final String CONFIG_PATH = "sdk-config/sdk.properties";

    @Override
    protected void configure() {

        bindConfigProperties();

        bind(CryptoService.class).to(JWECryptoService.class);
        bind(CertificateValidator.class).to(KeyValidator.class);
        bind(MetadataValidator.class).to(MetadataSubmissionValidator.class);
        bind(MetadataService.class).to(MetadataUploadService.class);

        bind(Sender.class).to(SubmissionSender.class);
        bind(Subscriber.class).to(SubmissionSubscriber.class);
    }

    @Provides
    OAuthService provideOAuthService(@Named("authTokenUrl") String tokenUrl, RestTemplate restTemplate) {
        return new DefaultOAuthService(restTemplate, tokenUrl);
    }

    @Provides
    @Singleton
    RestTemplate provideRestTemplate(@Named("httpProxyHost") String proxyHost, @Named("httpProxyPort") int proxyPort) {
        return new ProxyConfig(proxyHost, proxyPort).proxyRestTemplate();
    }

    private void bindConfigProperties() {
        try (InputStream in = new FileInputStream(CONFIG_PATH)) {
            final Properties properties = new Properties();
            properties.load(in);
            Names.bindProperties(binder(), properties);
            logger.info("using sdk config: {}", properties);
        } catch (IOException e) {
            logger.error("could not read sdk properties", e);
        }
    }
}
